# coding=utf-8
from itertools import chain


class Context(object):
    def __init__(self, args=None, context=None):
        self._args_spaces = []
        self._context_spaces = []

        self.add_args(tuple(args) if args is not None else ())
        self.update_context_with(context if context is not None else {})

    @property
    def args(self):
        return tuple(chain(*self._args_spaces))

    @property
    def kwargs(self):
        ctx = {}
        for c in reversed(self._context_spaces):
            ctx.update(c)
        return ctx

    def add_args(self, args):
        self._args_spaces.append(tuple(args))

    def update_context_with(self, context, update_initial=False):
        if not update_initial:
            self._context_spaces.insert(0, context.copy())
        else:
            self._context_spaces[-1].update(context)

    def clear(self, save_initial=True):
        if save_initial:
            del self._args_spaces[1:]
            del self._context_spaces[:-1]
        else:
            del self._args_spaces[:]
            del self._context_spaces[:]


__all__ = ['Context']


# coding=utf-8
"""
Provides some classes that allow to manipulate the context data flexibly and
get lazy object that renders itself only if it's used as a string.
"""

from lazy_format.aliases import Aliases
from lazy_format.context import Context
from lazy_format.formatter import LazyFormatter


class LazyFormat(object):

    _formatter = LazyFormatter()

    # shared attribute
    # uses the kwargs (context) functionality of the Context class only
    global_context = Context()

    def __init__(self, text, *partial_args, **partial_context):
        self._text = text
        self._context = Context(partial_args, dict(self.global_context.kwargs, **partial_context))

    def context(self, args, kwargs):
        self._context.clear()
        self._context.add_args(args)
        self._context.update_context_with(kwargs)
        return self._context

    def format(self, *args, **kwargs):
        context = self.context(args, kwargs)
        return self._formatter.format(self._text, *context.args, **context.kwargs)

    __format__ = __unicode__ = __str__ = format

    def __add__(self, other):
        return '{0!s}{1!s}'.format(self, other)

    def __eq__(self, other):
        return self.format() == other

    def __ne__(self, other):
        return self.format() != other

    def __len__(self):
        return len(self.format())


class LazyAliasFormat(LazyFormat):
    CONTEXT_ALIAS_NAMESPACE = 'alias'

    def __init__(self, text, *partial_args, **partial_context):
        alias = partial_context.pop('alias', None)

        super(LazyAliasFormat, self).__init__(text, *partial_args, **partial_context)

        self._alias = alias
        self._aliases = Aliases()

        if self._alias:
            self._aliases.add_alias(alias, self)

    def context(self, args, kwargs):
        context = super(LazyAliasFormat, self).context(args=args, kwargs=kwargs)
        context.update_context_with({
            self.CONTEXT_ALIAS_NAMESPACE: self._aliases.context(context)
        })
        return context


__all__ = ['LazyAliasFormat' , 'LazyFormat']



# coding=utf-8


class Aliases(object):
    _aliases = {}  # shared attribute

    def __init__(self, context=None):
        self.context(context=context)

    def __getattr__(self, key):
        from lazy_format.format import LazyFormat

        if key in self._aliases:
            value = self._aliases[key]

            if isinstance(value, LazyFormat):
                return value.format(*self._runtime_context.args, **self._runtime_context.kwargs)

            if callable(value):
                return value(alias=key, context=self._runtime_context)

            return value

        raise AttributeError(key)

    @classmethod
    def add_alias(cls, alias, value):
        cls._aliases[alias] = value

    @classmethod
    def remove_alias(cls, alias):
        cls._aliases.pop(alias, None)

    @classmethod
    def clear_aliases(cls):
        cls._aliases.clear()

    def context(self, context=None):
        from lazy_format.context import Context
        self._runtime_context = context if context is not None else Context()
        return self


__all__ = ['Aliases']



from distutils.core import setup

setup(
    name='lazy_format',
    version='0.0.2',
    packages=['lazy_format'],
    url='',
    license='LGPL',
    author='Miphreal Adler',
    author_email='miphreal@gmail.com',
    description=''
)



# coding=utf-8
import string


class LazyFormatter(string.Formatter):
    def _unsplit_var(self, conversion, field_name, format_spec):
        variable = (
            '{',
            field_name,
            (('!' + conversion) if conversion else ''),
            ((':' + format_spec) if format_spec else ''),
            '}'
        )
        return ''.join(variable)

    def _vformat(self, format_string, args, kwargs, used_args, recursion_depth):
        if recursion_depth < 0:
            raise ValueError('Max string recursion exceeded')
        result = []
        for literal_text, field_name, format_spec, conversion in \
                self.parse(format_string):

            if literal_text:
                result.append(literal_text)

            if field_name is not None:
                try:
                    obj, arg_used = self.get_field(field_name, args, kwargs)
                    used_args.add(arg_used)
                    format_spec = self._vformat(format_spec, args, kwargs,
                                                used_args, recursion_depth - 1)
                    rendered = self.format_field(self.convert_field(obj, conversion), format_spec)
                except (KeyError, IndexError):
                    format_spec = self._vformat(format_spec, args, kwargs,
                                                used_args, recursion_depth - 1)
                    rendered = self._unsplit_var(conversion, field_name, format_spec)

                result.append(rendered)

        return ''.join(result)


__all__ = ['LazyFormatter']